//LTC2320 Verilog driver for Altera Cyclone III FPGA
//Data format from ADC is CMOS
//This version is 8 differential channels, internally clocked
//There are 8 serial CMOS output data streams
//New ADC package format
//Input clock rate (52.5MHz) is 35 times the output clock rate of 1.5Msps
//Note that D[] bits are read on ch 1 of PSCOPE
//The timing constraints for this device are:
// CNV pulse width >30nsec
// Time from CNV falling edge to MSB on SDOx line >450nsec
//DJS 6_20_16

`timescale 1ps / 1ps
module LTC2320_cmos
(	
	CLKIN,
	SCK,SCK_GND,
	CLKOUT,CLKOUT_GND,
	CNVCLK,
	CNV_EN,
	SDO1,SDO2,SDO3,SDO4,SDO5,SDO6,SDO7,SDO8,
	D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15,
	DDR,
	BITSEL1,BITSEL2,
	FR0,FR1,FR2,
	sync
);

input CLKIN;												//Clock in from user @ 52.5 MHz
input	CLKOUT;												//CLKOUT signal from the ADC
output CLKOUT_GND;
input	SDO1,SDO2,SDO3,SDO4,SDO5,SDO6,SDO7,SDO8;	//Data out Chs 1,2,3,4,5,6,7,8 from ADC
input DDR;													//Control signal from demo board; use DDR mode when 'high'
input BITSEL1,BITSEL2;
output CNVCLK;												//Latch signal from FPGA to DC890 on falling edge
output CNV_EN;												//Convert start on falling edge							
output SCK,SCK_GND;										//SCK from FPGA to ADC @ 105/52.5 MHz SDR/DDR
output D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15;
output FR0,FR1,FR2;
output sync;

reg SCK_GND;
reg CLKOUT_GND;	
reg [15:0] D;
reg CNVCLK;	
reg [3:0] channel;
reg FR0,FR1,FR2;

integer num_bits;

wire CNV_EN;
wire sck_gate,sync,rx_start_p,rx_start_n,data_latch;						
wire CLKOUT;
wire clk,clk_52p5,rx_clk_52p5,rx_clk_105,clk_105;
wire SDO1,SDO2,SDO3,SDO4,SDO5,SDO6,SDO7,SDO8;
wire [15:0] data [7:0];

assign D0 = D[0];
assign D1 = D[1];
assign D2 = D[2];
assign D3 = D[3];
assign D4 = D[4];
assign D5 = D[5];
assign D6 = D[6];
assign D7 = D[7];
assign D8 = D[8];
assign D9 = D[9];
assign D10 = D[10];
assign D11 = D[11];
assign D12 = D[12];
assign D13 = D[13];
assign D14 = D[14];
assign D15 = D[15];

initial
begin
	SCK_GND = 0;
	CLKOUT_GND = 0;
	CNVCLK = 0;
end

//'sync' pulse is 60nsec wide
pll_sync synchronize(
	.inclk0(CLKIN),
	.c0(clk_52p5),
	.c1(rx_clk_52p5),
	.c2(clk_105),
	.c3(rx_clk_105),
	.c4(sync)
);

//select clocks for SDR/DDR mode
assign clk = (DDR & clk_52p5)|(~DDR & clk_105);

//generate the 'SCK' signal
altddioout	sckddr 
( 
	.datain_h(sck_gate),
	.datain_l(1'b0),
	.outclock(clk),
	.dataout(SCK)
);

always @(negedge clk)
begin
//decode the number of bits
//Note: the number of bits may not be relevant or necessary is the DC890 left justifies the parallel bus (LSB's are zero for num_bits < 16)
	if(BITSEL1 == 1)
    begin
      if(BITSEL2 == 1)  //16 bits
			    num_bits = 16;
      else  //14 bits
		      num_bits = 14;	//note the code has to clock out 16 bits for the 14 bit part
	  end
	else  //12 bits
	   num_bits = 12;	//note the code has to clock out 16 bits for the 12 bit part
end
	
//generate the timing pulses 'sck_gate', 'CNV_EN', 'CNVCLK', and the 'channel'
pulse_generator pulses
(
	.clk(clk_105),
	.sync(sync),
	.ddr(DDR),
	.cnv_en(CNV_EN),
	.cnvclk_gate(cnvclk_gate),
	.sck_gate(sck_gate),
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.data_latch(data_latch)
);

// Receive serial data from ADC ch 1 and converts it to parallel format
deserializer rx_1
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_105),
	.clk_ddr(rx_clk_52p5),
	.sdo(SDO1),
	.parallel_data(data[0]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC ch 2 and converts it to parallel format
deserializer rx_2
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_105),
	.clk_ddr(rx_clk_52p5),
	.sdo(SDO2),
	.parallel_data(data[1]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

//// Receive serial data from ADC ch 3 and converts it to parallel format
deserializer rx_3
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_105),
	.clk_ddr(rx_clk_52p5),
	.sdo(SDO3),
	.parallel_data(data[2]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC ch 4 and converts it to parallel format
deserializer rx_4
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_105),
	.clk_ddr(rx_clk_52p5),
	.sdo(SDO4),
	.parallel_data(data[3]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC ch 5 and converts it to parallel format
deserializer rx_5
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_105),
	.clk_ddr(rx_clk_52p5),
	.sdo(SDO5),
	.parallel_data(data[4]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC ch 6 and converts it to parallel format
deserializer rx_6
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_105),
	.clk_ddr(rx_clk_52p5),
	.sdo(SDO6),
	.parallel_data(data[5]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC ch 7 and converts it to parallel format
deserializer rx_7
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_105),
	.clk_ddr(rx_clk_52p5),
	.sdo(SDO7),
	.parallel_data(data[6]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC ch 8 and converts it to parallel format
deserializer rx_8
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_105),
	.clk_ddr(rx_clk_52p5),
	.sdo(SDO8),
	.parallel_data(data[7]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

//generate the 'CNVCLK' signal
always @ (posedge clk_52p5)
begin
  if(~cnvclk_gate)
    CNVCLK = 0;
  else
    CNVCLK = ~CNVCLK; 
end

//generate the channel index
always @(negedge CNVCLK or negedge cnvclk_gate)
begin	
		if(~cnvclk_gate)	
			channel = 0;	
		else if(channel == 4'b111)
			channel = 0; 
		else
			channel = channel + 1;
end

always @ (posedge CNVCLK)
begin
	//load the parallel data from one channel onto the 16 bit bus
   D = data[channel];
	
	//decode the frame bits
	FR0 = 4'b0001 & (channel);
	FR1 = (4'b0010 & (channel)) >> 1;
	FR2 = (4'b0100 & (channel)) >> 2;
end

endmodule

