//LTC2320 Verilog driver for Altera Cyclone III FPGA
//Data format from ADC is LVDS
//This version is 8 differential channels, internally clocked
//There are 4 serial LVDS output data streams
//New ADC package format
//Input clock rate (75MHz) is 60 times the output clock rate of 1.5Msps
//The timing constraints for this device are:
// CNV pulse width >30nsec
// Time from CNV falling edge to MSB on SDOx line >450nsec
//DJS 6_15_16


`timescale 1ps / 1ps
module LTC2320_lvds
(	
	CLKIN,
	SCKn,
	CLKOUTn,
	CNVCLK,
	CNV_EN,
	SDO_An,SDO_Bn,SDO_Cn,SDO_Dn,
	D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15,
	DDR,
	BITSEL1,BITSEL2,
	FR0,FR1,FR2,
	sync
);

input CLKIN;								//Clock in from user @ 75 MHz
input	CLKOUTn;								//CLKOUT signal from the ADC
input	SDO_An,SDO_Bn,SDO_Cn,SDO_Dn;	//Data out Chs 1,2,3,4,5,6,7,8 from ADC
input DDR;									//Control signal from demo board; use DDR mode when 'high'
input BITSEL1,BITSEL2;

output CNVCLK;								//Latch signal from FPGA to DC890 on falling edge
output CNV_EN;								//Convert start on falling edge							
output SCKn;								//SCK from FPGA to ADC @ 300/150 MHz SDR/DDR
output D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15;
output FR0,FR1,FR2;
output sync;

reg [15:0] D;
reg CNVCLK;	
reg [3:0] channel;
reg FR0,FR1,FR2;

integer num_bits;

wire CNV_EN;
wire sck_gate,cnvclk_gate,sync,rx_start_p,rx_start_n,data_latch;							
wire CLKOUTn;
wire clk,clk_150,rx_clk_150,clk_300,rx_clk_300;
wire SDO_An,SDO_Bn,SDO_Cn,SDO_Dn;
wire [15:0] data [7:0];

assign D0 = D[0];
assign D1 = D[1];
assign D2 = D[2];
assign D3 = D[3];
assign D4 = D[4];
assign D5 = D[5];
assign D6 = D[6];
assign D7 = D[7];
assign D8 = D[8];
assign D9 = D[9];
assign D10 = D[10];
assign D11 = D[11];
assign D12 = D[12];
assign D13 = D[13];
assign D14 = D[14];
assign D15 = D[15];

initial
	CNVCLK = 0;

//'sync' pulse is 60nsec wide
pll_sync synchronize(
	.inclk0(CLKIN),
	.c0(clk_150),
	.c1(rx_clk_150),
	.c2(clk_300),
	.c3(rx_clk_300),
	.c4(sync)
);

//select clocks for SDR/DDR mode
assign clk = (DDR & clk_150)|(~DDR & clk_300);

//generate the 'SCKn' signal
altddioout	sckddr 
( 
	.datain_h(~sck_gate),
	.datain_l(1'b1),
	.outclock(clk),
	.dataout(SCKn)
);

always @(negedge clk)
begin
//decode the number of bits
//Note: the number of bits may not be relevant or necessary is the DC890 left justifies the parallel bus (LSB's are zero for num_bits < 16)
	if(BITSEL1 == 1)
    begin
      if(BITSEL2 == 1)  //16 bits
			    num_bits = 16;
      else  //14 bits
		      num_bits = 14;	//note the code has to clock out 16 bits for the 14 bit part
	  end
	else  //12 bits
	   num_bits = 12;	//note the code has to clock out 16 bits for the 12 bit part
end
	
//generate the timing pulses 'sck_gate', 'CNV_EN', 'CNVCLK', and the 'channel'
pulse_generator pulses
(
	.clk(clk_300),
	.sync(sync),
	.ddr(DDR),
	.cnv_en(CNV_EN),
	.cnvclk_gate(cnvclk_gate),
	.sck_gate(sck_gate),
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.data_latch(data_latch)
);

// Receive serial data from ADC chs 1,2 and converts it to parallel format
deserializer rx_12
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_300),
	.clk_ddr(rx_clk_150),
	.sdo(~SDO_An),
	.parallel_data_1(data[0]),
	.parallel_data_2(data[1]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC chs 3,4 and converts it to parallel format
deserializer rx_34
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_300),
	.clk_ddr(rx_clk_150),
	.sdo(~SDO_Bn),
	.parallel_data_1(data[2]),
	.parallel_data_2(data[3]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC chs 5,6 and converts it to parallel format
deserializer rx_56
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_300),
	.clk_ddr(rx_clk_150),
	.sdo(~SDO_Cn),
	.parallel_data_1(data[4]),
	.parallel_data_2(data[5]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

// Receive serial data from ADC chs 7,8 and converts it to parallel format
deserializer rx_78
(
	.rx_start_p(rx_start_p),
	.rx_start_n(rx_start_n),
	.clk_msb(sck_gate),
	.clk_sdr(rx_clk_300),
	.clk_ddr(rx_clk_150),
	.sdo(~SDO_Dn),
	.parallel_data_1(data[6]),
	.parallel_data_2(data[7]),
	.data_latch(data_latch),
	.ddr(DDR),
	.num_bits(num_bits)
);

//generate the 'CNVCLK' signal
always @ (posedge clk_150)
begin
  if(~cnvclk_gate)
    CNVCLK = 0;
  else
    CNVCLK = ~CNVCLK; 
end

//generate the channel index
always @(negedge CNVCLK or negedge cnvclk_gate)
begin	
		if(~cnvclk_gate)	
			channel = 0;	
		else if(channel == 4'b111)
			channel = 0; 
		else
			channel = channel + 1;
end

always @ (posedge CNVCLK)
begin
	//load the parallel data from one channel onto the 16 bit bus
   D = data[channel];
	
	//decode the frame bits
	FR0 = 4'b0001 & (channel);
	FR1 = (4'b0010 & (channel)) >> 1;
	FR2 = (4'b0100 & (channel)) >> 2;
end

endmodule

